// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt;

public class CRFMath {
    public static final double LOGZERO = Double.NaN;
    public static final double LOGONE = 0;

    public static final boolean isLogZero(double x) {
        return Double.isNaN(x);
    }

    public static final double eExp(double x) {
        return isLogZero(x) ? 0 : Math.exp(x);
    }

    public static final double eLog(double x) {
        if(x == 0)
            return LOGZERO;
        else if(x > 0)
            return Math.log(x);
        else
            throw new IllegalArgumentException();
    }

    public static final double eLogSum(double elogx, double elogy) {
        if(isLogZero(elogx))
            return elogy;
        else if(isLogZero(elogy))
            return elogx;
        else if(elogx > elogy)
            return elogx + eLog(1.0 + Math.exp(elogy - elogx));
        else
            return elogy + eLog(1.0 + Math.exp(elogx - elogy));
    }

    public static final double eLogProduct(double elogx, double elogy) {
        if(isLogZero(elogx) || isLogZero(elogy))
            return LOGZERO;
        else
            return elogx + elogy;
    }
}
