// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt;

import com.pawjaw.classification.crf.lmcbt.configurations.Configuration;
import com.pawjaw.classification.crf.lmcbt.points.Point;
import com.pawjaw.classification.crf.lmcbt.sequences.TrainingSequences;
import com.pawjaw.classification.crf.lmcbt.trees.BoostedTrees;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Semaphore;

public class CRFValidator {
    private double training_percent;
    private Configuration c;
    private Semaphore proceed;
    private TrainingSequences ts;
    private List<Point[]> all_pointss;
    private List<int[]> all_true_labelss;
    private Semaphore update_complete = new Semaphore(0);
    private List<Integer> example_indexes = new ArrayList();
    private List<Point[]> validation_points = new ArrayList();
    private List<int[]> validation_true_labels = new ArrayList();
    private BoostedTrees[] bt_k;

    public List<Double> accuracies = new ArrayList();
    public List<Integer> boosting_iterations = new ArrayList();

    public CRFValidator(Configuration c, List<Point[]> pointss, List<int[]> true_labelss) {
        int i, I = pointss.size();
        this.c = c;
        this.all_pointss = pointss;
        this.all_true_labelss = true_labelss;
        proceed = new Semaphore(c.boosted_tree_threads);
        bt_k = new BoostedTrees[c.label_count_including_start_label];
        for(i = 0;i < I;i++)
            example_indexes.add(i);
    }
    
    public void run(int cross_validation_passes, double training_percent) {
        int i;
        this.training_percent = training_percent;
        for(i = 0;i < cross_validation_passes;i++) {
            setupPass();
            runPass(i);
        }
    }

    private void setupPass() {
        int i = 0, I, J = (int)Math.ceil((double)(I = example_indexes.size()) * training_percent);
        int k, K = bt_k.length, example_index;
        Collections.sort(example_indexes);
        ts = new TrainingSequences();
        ts.allocate(c, J);
        validation_points.clear();
        validation_true_labels.clear();
        while(i < J) {
            example_index = example_indexes.get(i);
            ts.set(i, all_pointss.get(example_index), all_true_labelss.get(example_index));
            i++;
        }
        while(i < I) {
            example_index = example_indexes.get(i);
            validation_points.add(all_pointss.get(example_index));
            validation_true_labels.add(all_true_labelss.get(example_index));
            i++;
        }
        ts.indexExamples();
        if(c.cache_expanded_true_features)
            ts.cacheTrueFeatures();
        for(k = 0;k < K;k++)
            (bt_k[k] = new BoostedTrees(c, ts, k, proceed, update_complete)).start();
    }

    private double[][][] serialize() {
        int k, K = bt_k.length;
        double[][][] sbt_k = new double[K][][];
        for(k = 0;k < K;k++)
            sbt_k[k] = bt_k[k].serialize();
        return sbt_k;
    }

    private void runPass(int cross_validation_pass) {
        int k, K = bt_k.length, m;
        double accuracy;
        CRFInference crfi = new CRFInference();
        for(m = 1;m <= c.max_boosting_iterations;m++) {
            crfi.set(c, serialize());
            boosting_iterations.add(m);
            accuracies.add(accuracy = crfi.accuracyViterbi(validation_points, validation_true_labels));
            System.out.println("cross validation pass: " + cross_validation_pass + ", iterations: " + m + ", accuracy: " + accuracy);
            ts.update();
            for(k = 0;k < K;k++)
                bt_k[k].setUpdateable();
            try {
                update_complete.acquire(K);
            } catch(InterruptedException e) {
                break;
            }
        }
        for(k = 0;k < K;k++)
            bt_k[k].finished();
    }
}
