// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.points;

import com.pawjaw.classification.crf.lmcbt.configurations.Configuration;
import java.util.Arrays;

public class SparsePoint implements Point {
    private int[] posfs;

    public void setFeatures(boolean[] bs) {
        int f, F = bs.length, n = 0, N = 0;
        for(f = 0;f < F;f++)
            if(bs[f])
                N++;
        posfs = new int[N];
        for(f = 0;f < F;f++)
            if(bs[f])
                posfs[n++] = f;
    }

    public void setFeatures(int[] positive_features) {
        posfs = positive_features;
        Arrays.sort(posfs);
    }

    public boolean hasTrueFeature(Configuration c, int feature_index) {
        int l = 0, h = posfs.length - 1, m;
        while(l <= h) {
            if(feature_index < posfs[m = (l + h) / 2])
                h = m - 1;
            else if(feature_index > posfs[m])
                l = m + 1;
            else
                return true;
        }
        return false;
    }

    public int nextTrueFeature(Configuration c, int offset) {
        if(offset < 0 || offset >= posfs.length)
            return -1;
        else
            return posfs[offset];
    }

    public int trueFeatureCount() {
        return posfs.length;
    }
}
