// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.readers;

import com.pawjaw.classification.crf.lmcbt.points.Point;
import com.pawjaw.classification.crf.lmcbt.points.SparsePoint;
import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Pattern;

public class CoNLL2000LabeledSequenceDataReader extends LabeledSequenceDataReader {
    private List<String> tokens = new ArrayList();
    private List<String> labels = new ArrayList();
    private Map<String, Integer> idx_token = new TreeMap();
    private Map<String, Integer> idx_label = new TreeMap();
    private Pattern splitter = Pattern.compile("\\s+");

    public int getPointFeatureCount() {
        return tokens.size();
    }

    public int getLabelCount() {
        return labels.size();
    }

    private class LabeledPoint {
        public int label_idx;
        public SparsePoint sp = new SparsePoint();
    }

    private int getTokenIdx(String token) {
        Integer idx = idx_token.get(token);
        if(idx == null) {
            idx_token.put(token, idx = tokens.size());
            tokens.add(token);
        }
        return idx;
    }

    private int getLabelIdx(String label) {
        Integer idx = idx_label.get(label);
        if(idx == null) {
            idx_label.put(label, idx = labels.size());
            labels.add(label);
        }
        return idx;
    }

    private LabeledPoint readNextWord(BufferedReader br) throws IOException {
        String s = br.readLine();
        LabeledPoint lp = new LabeledPoint();
        String[] ss;
        if(s == null)
            throw new EOFException();
        if((s = s.trim()).isEmpty() || (ss = splitter.split(s)).length != 3)
            return null;
        lp.sp.setFeatures(new int[] { getTokenIdx(ss[0]), getTokenIdx("POS_" + ss[1]) });
        lp.label_idx = getLabelIdx(ss[2]);
        return lp;
    }

    private void readNextSentence(BufferedReader br, List<LabeledPoint> lps) throws IOException {
        LabeledPoint lp;
        try {
            while((lp = readNextWord(br)) != null)
                lps.add(lp);
        } catch(EOFException e) {
            if(lps.isEmpty())
                throw e;
        }
    }

    public void readPoints(List<Point[]> pointss, List<int[]> true_labelss, BufferedReader br) throws IOException {
        int i, I;
        int[] is;
        Point[] ps;
        List<LabeledPoint> lps = new ArrayList();
        try {
            while(true) {
                readNextSentence(br, lps);
                if((I = lps.size()) != 0) {
                    is = new int[I];
                    ps = new Point[I];
                    for(i = 0;i < I;i++) {
                        ps[i] = lps.get(i).sp;
                        is[i] = lps.get(i).label_idx;
                    }
                    pointss.add(ps);
                    true_labelss.add(is);
                }
                lps.clear();
            }
        } catch(EOFException e) {}
    }
}
