// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.readers;

import com.pawjaw.classification.crf.lmcbt.points.Point;
import com.pawjaw.classification.crf.lmcbt.points.SparsePoint;
import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

public class TreeCRFLabeledSequenceDataReader extends LabeledSequenceDataReader {
    private int max_point = -1, max_label = -1;
    private int previous_sequence = -1;
    private Pattern splitter = Pattern.compile("\\s+");

    private void readPoint(List<List<Point>> pointss, List<List<Integer>> true_labelss, BufferedReader br) throws IOException {
        int k, K, sequence, label;
        String s = br.readLine();
        List<Point> points;
        List<Integer> labels;
        SparsePoint sp = new SparsePoint();
        String[] ss;
        int[] feature_indexes;
        if(s == null)
            throw new EOFException();
        if((ss = splitter.split(s)).length >= 5) {
            feature_indexes = new int[K = Integer.parseInt(ss[2])];
            for(k = 0;k < K;k++)
                if((feature_indexes[k] = Integer.parseInt(ss[k + 3])) > max_point)
                    max_point = feature_indexes[k];
            if((sequence = Integer.parseInt(ss[0])) != previous_sequence) {
                pointss.add(points = new ArrayList());
                true_labelss.add(labels = new ArrayList());
                previous_sequence = sequence;
            } else {
                points = pointss.get(pointss.size() - 1);
                labels = true_labelss.get(true_labelss.size() - 1);
            }
            sp.setFeatures(feature_indexes);
            points.add(sp);
            labels.add(label = Integer.parseInt(ss[ss.length - 1]));
            if(label > max_label)
                max_label = label;
        }
    }

    public void readPoints(List<Point[]> pointss, List<int[]> true_labelss, BufferedReader br) throws IOException {
        int i, I, k, K;
        List<List<Point>> _pointss = new ArrayList();
        List<List<Integer>> _true_labelss = new ArrayList();
        Point[] points;
        int[] labels;
        br.readLine();
        try {
            while(true)
                readPoint(_pointss, _true_labelss, br);
        } catch(EOFException e) {}
        K = _pointss.size();
        for(k = 0;k < K;k++) {
            points = new Point[I = _pointss.get(k).size()];
            labels = new int[I];
            for(i = 0;i < I;i++) {
                points[i] = _pointss.get(k).get(i);
                labels[i] = _true_labelss.get(k).get(i);
            }
            pointss.add(points);
            true_labelss.add(labels);
        }
    }

    public int getPointFeatureCount() {
        return max_point + 1;
    }

    public int getLabelCount() {
        return max_label + 1;
    }
}
