// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.sequences;

import com.pawjaw.classification.crf.lmcbt.configurations.Configuration;
import com.pawjaw.classification.crf.lmcbt.points.Point;
import java.util.List;

public class ExpandedPointSequence {
    private Point[] sp_t;

    public void set(Point[] sp_t) {
        this.sp_t = sp_t;
    }

    public int length() {
        return sp_t.length;
    }

    public int expandedLength(Configuration c) {
        return sp_t.length * c.label_count_including_start_label;
    }

    public boolean hasTrueFeature(Configuration c, int sequence_position, int previous_label, int feature_index) {
        if(feature_index < c.label_count_including_start_label)
            return previous_label == feature_index;
        sequence_position += (feature_index - c.label_count_including_start_label) /
                c.point_features - c.window_radius;
        if(sequence_position < 0 || sequence_position >= sp_t.length)
            return false;
        else if(feature_index < c.valid_window_offset)
            return sp_t[sequence_position].hasTrueFeature(c,
                    (feature_index - c.label_count_including_start_label) %
                    c.point_features);
        else
            return true;
    }

    public boolean hasTrueFeature(Configuration c, int expanded_sequence_position, int feature_index) {
        return hasTrueFeature(c, expanded_sequence_position / c.label_count_including_start_label,
                expanded_sequence_position % c.label_count_including_start_label,
                feature_index);
    }

    public List<Integer> getTrueFeatures(Configuration c, int sequence_position, int previous_label, List<Integer> feature_indexes) {
        int t, T = length(), f, o, fo = c.label_count_including_start_label, w, W = c.window_radius * 2 + 1;
        feature_indexes.clear();
        feature_indexes.add(previous_label);
        sequence_position -= c.window_radius;
        for(w = 0;w < W;w++) {
            if((t = sequence_position + w) >= 0 && t < T) {
                o = 0;
                while((f = sp_t[t].nextTrueFeature(c, o++)) >= 0)
                    feature_indexes.add(f + fo);
            }
            fo += c.point_features;
        }
        for(w = 0;w < W;w++)
            if((t = sequence_position + w) >= 0 && t < T)
                feature_indexes.add(fo + w);
        return feature_indexes;
    }

    public List<Integer> getTrueFeatures(Configuration c, int expanded_sequence_position, List<Integer> feature_indexes) {
        return getTrueFeatures(c, expanded_sequence_position / c.label_count_including_start_label,
                expanded_sequence_position % c.label_count_including_start_label,
                feature_indexes);
    }
}
