// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.sequences;

import com.pawjaw.classification.crf.lmcbt.configurations.Configuration;
import com.pawjaw.classification.crf.lmcbt.points.Point;
import java.util.ArrayList;
import java.util.List;

public class TrainingSequences {
    private Configuration c;
    private TrainingSequence[] trainingsequence_s;
    private int[][] s_offset_i;
    private int[][] true_features_i;

    public void allocate(Configuration c, int training_sequences) {
        this.c = c;
        trainingsequence_s = new TrainingSequence[training_sequences];
    }

    public int length() {
        return trainingsequence_s.length;
    }

    public void set(int sequence_index, Point[] points, int[] labels) {
        trainingsequence_s[sequence_index] = new TrainingSequence();
        trainingsequence_s[sequence_index].set(points);
        trainingsequence_s[sequence_index].allocate(c);
        trainingsequence_s[sequence_index].setLabels(labels);
    }

    public void indexExamples() {
        int i = 0, s = 0, I = 0, x, X, offset = 0;
        for(TrainingSequence ts : trainingsequence_s)
            I += ts.length() * c.label_count_including_start_label;
        s_offset_i = new int[I][2];
        for(TrainingSequence ts : trainingsequence_s) {
            X = ts.length() * c.label_count_including_start_label;
            for(x = 0;x < X;x++) {
                s_offset_i[i][0] = s;
                s_offset_i[i][1] = offset;
                i++;
            }
            s++;
            offset += X;
        }
    }

    public void cacheTrueFeatures() {
        int example_index, examples = examples(), i;
        List<Integer> feature_indexes = new ArrayList();
        true_features_i = new int[examples][];
        for(example_index = 0;example_index < examples;example_index++) {
            i = 0;
            getTrueFeatures(example_index, feature_indexes);
            true_features_i[example_index] = new int[feature_indexes.size()];
            for(int feature_index : feature_indexes)
                true_features_i[example_index][i++] = feature_index;
        }
    }

    public int examples() {
        return s_offset_i.length;
    }

    public int features() {
        return c.expanded_features;
    }

    public boolean hasTrueFeature(int example_index, int feature_index) {
        return trainingsequence_s[s_offset_i[example_index][0]].hasTrueFeature(c,
                example_index - s_offset_i[example_index][1],
                feature_index);
    }

    public List<Integer> getTrueFeatures(int example_index, List<Integer> feature_indexes) {
        return trainingsequence_s[s_offset_i[example_index][0]].getTrueFeatures(c,
                example_index - s_offset_i[example_index][1],
                feature_indexes);
    }

    public int[] getCachedTrueFeatures(int example_index) {
        return true_features_i[example_index];
    }

    public int compare(int example_indexa, int example_indexb, int feature_index) {
        boolean a = hasTrueFeature(example_indexa, feature_index);
        boolean b = hasTrueFeature(example_indexb, feature_index);
        return ((a && b) || (!a && !b)) ? 0 : (!a && b) ? -1 : 1;
    }

    public double getTarget(int label, int example_index) {
        return trainingsequence_s[s_offset_i[example_index][0]].getTarget(label, example_index - s_offset_i[example_index][1]);
    }

    public void incrementPotential(int label, int example_index, double potential) {
        trainingsequence_s[s_offset_i[example_index][0]].incrementPotential(label, example_index - s_offset_i[example_index][1], potential);
    }

    public void update() {
        for(TrainingSequence ts : trainingsequence_s) {
            ts.updateForwardBackward();
            ts.updateTargets();
        }
    }
}
