// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.tests;

import com.pawjaw.classification.crf.lmcbt.CRFValidator;
import com.pawjaw.classification.crf.lmcbt.configurations.Configuration;
import com.pawjaw.classification.crf.lmcbt.configurations.DefaultConfiguration;
import com.pawjaw.classification.crf.lmcbt.points.Point;
import com.pawjaw.classification.crf.lmcbt.readers.LabeledSequenceDataReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public abstract class CRFTest {
    public abstract int getWindowRadius();
    public abstract LabeledSequenceDataReader getLabeledSequenceDataReader();

    public int getCrossValidationPasses() {
        return 8;
    }

    public double getTrainingPercentage() {
        return 0.8;
    }

    public List<Double> accuracies;
    public List<Integer> boosting_iterations ;

    public void test(File sequence_data_file) throws IOException {
        CRFValidator v;
        Configuration c;
        List<Point[]> pointss = new ArrayList();
        List<int[]> true_labelss = new ArrayList();
        LabeledSequenceDataReader r = getLabeledSequenceDataReader();
        r.readFile(pointss, true_labelss, sequence_data_file);
        c = new DefaultConfiguration(r.getPointFeatureCount(), getWindowRadius(), r.getLabelCount());
        v = new CRFValidator(c, pointss, true_labelss);
        v.run(getCrossValidationPasses(), getTrainingPercentage());
        accuracies = v.accuracies;
        boosting_iterations = v.boosting_iterations;
    }
}
