// Copyright 2012, 2013 Brad Block, Pawjaw, LLC. (an Ohio Limited Liability Company)
// 
// This file is part of JBTCRF.
// 
// JBTCRF is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// JBTCRF is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with JBTCRF.  If not, see <http://www.gnu.org/licenses/>.

package com.pawjaw.classification.crf.lmcbt.trees;

import com.pawjaw.classification.crf.lmcbt.configurations.Configuration;
import com.pawjaw.classification.crf.lmcbt.sequences.ExpandedPointSequence;
import com.pawjaw.classification.crf.lmcbt.sequences.TrainingSequences;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.Semaphore;

public class BoostedTrees extends Thread {
    private int label;
    private int examples;
    private Tree tree;
    private Configuration c;
    private Splitter splitter;
    private TrainingSequences ts;
    private Semaphore updateable, proceed, update_complete;
    private List<double[]> serialized_trees = new ArrayList();
    private int[] training_examples;

    public BoostedTrees(Configuration c, TrainingSequences ts, int label, Semaphore proceed, Semaphore update_complete) {
        int i;
        this.c = c;
        this.ts = ts;
        this.label = label;
        this.proceed = proceed;
        this.update_complete = update_complete;
        tree = new Tree();
        examples = ts.examples();
        updateable = new Semaphore(0);
        splitter = new Splitter(c, ts);
        training_examples = new int[examples];
        for(i = 0;i < examples;i++)
            training_examples[i] = i;
    }

    private void buildTree() {
        tree.build(c, label, splitter, training_examples);
        serialized_trees.add(tree.serialize());
    }

    private void updatePotentials() {
        Node node;
        Queue<Node> nodeq = new LinkedList();
        nodeq.add(tree.root);
        while((node = nodeq.poll()) != null) {
            if(node.isLeaf())
                for(int example_index : node.example_indexes)
                    ts.incrementPotential(label, example_index, node.output);
            else {
                nodeq.add(node.true_child);
                nodeq.add(node.false_child);
            }
        }
    }

    public void setUpdateable() {
        updateable.release();
    }

    public void finished() {
        splitter.finished();
        this.interrupt();
    }

    public void run() {
        try {
            while(true) {
                updateable.acquire();
                proceed.acquire();
                buildTree();
                updatePotentials();
                proceed.release();
                update_complete.release();
            }
        } catch(InterruptedException e) {}
    }

    public double[][] serialize() {
        int i = 0;
        double[][] serialized_trees = new double[this.serialized_trees.size()][];
        for(double[] serialized_tree : this.serialized_trees)
            serialized_trees[i++] = serialized_tree;
        return serialized_trees;
    }

    public static double boostedOutput(Configuration c, ExpandedPointSequence eps, int expanded_sequence_position, double[][] serialized_trees) {
        double potential = 0;
        for(double[] serialized_tree : serialized_trees)
            potential += Tree.output(c, eps, expanded_sequence_position, serialized_tree);
        return potential;
    }

    public static double boostedOutput(Configuration c, ExpandedPointSequence eps, int sequence_position, int previous_label, double[][] serialized_trees) {
        double potential = 0;
        for(double[] serialized_tree : serialized_trees)
            potential += Tree.output(c, eps, sequence_position, previous_label, serialized_tree);
        return potential;
    }
}
